@@ -1,9 +1,8 @@
 use catvec::CatVec;
 use ethnum::U256;
+use themelio_structs::{CoinData, CoinDataHeight, CoinID, Denom, Header, Transaction};
 use tmelcrypt::HashVal;
 
-use crate::{CoinData, CoinDataHeight, CoinID, Denom, Header, HexBytes, Transaction};
-
 use super::Covenant;
 
 #[derive(PartialEq, Eq, Clone, Debug)]
@@ -156,12 +155,6 @@ impl From<Vec<u8>> for Value {
     }
 }
 
-impl From<HexBytes> for Value {
-    fn from(v: HexBytes) -> Self {
-        Value::Bytes(v.0.into())
-    }
-}
-
 impl<T: Into<Value>> From<Vec<T>> for Value {
     fn from(v: Vec<T>) -> Self {
         Value::Vector(
