@@ -1,25 +1,24 @@
 mod applytx;
+mod coins;
 pub(crate) mod melmint;
 pub(crate) mod melswap;
 mod poolkey;
 
+use crate::smtmapping::*;
 pub use crate::stake::*;
 use crate::state::applytx::StateHandle;
-use crate::{constants::*, melvm::Address, preseal_melmint, CoinDataHeight, Denom, TxHash};
-use crate::{smtmapping::*, BlockHeight, CoinData, CoinValue};
-use crate::{transaction::Transaction, CoinID};
 
+use std::io::Read;
 use std::{collections::BTreeMap, convert::TryInto};
-use std::{collections::BTreeSet, io::Read};
 use std::{collections::HashSet, fmt::Debug};
 
-use arbitrary::Arbitrary;
 use defmac::defmac;
 use derivative::Derivative;
 use novasmt::ContentAddrStore;
-use num_enum::{IntoPrimitive, TryFromPrimitive};
-use serde::{Deserialize, Serialize};
-use serde_repr::{Deserialize_repr, Serialize_repr};
+use themelio_structs::{
+    Address, Block, BlockHeight, CoinData, CoinDataHeight, CoinID, CoinValue, Denom, Header, NetID,
+    ProposerAction, Transaction, TxHash, STAKE_EPOCH, TIP_901_HEIGHT, TIP_902_HEIGHT,
+};
 use thiserror::Error;
 use tmelcrypt::{Ed25519PK, HashVal};
 
@@ -27,6 +26,8 @@ use crate::state::melswap::PoolMapping;
 
 pub use poolkey::PoolKey;
 
+pub use self::coins::CoinMapping;
+
 #[derive(Error, Debug)]
 /// A error that happens while applying a transaction to a state
 pub enum StateError {
@@ -54,33 +55,6 @@ pub enum StateError {
     DuplicateTx,
 }
 
-/// Identifies a network.
-#[derive(
-    Clone,
-    Copy,
-    IntoPrimitive,
-    TryFromPrimitive,
-    Eq,
-    PartialEq,
-    Debug,
-    Serialize_repr,
-    Deserialize_repr,
-    Hash,
-    Arbitrary,
-)]
-#[repr(u8)]
-pub enum NetID {
-    Testnet = 0x01,
-    Custom02 = 0x02,
-    Custom03 = 0x03,
-    Custom04 = 0x04,
-    Custom05 = 0x05,
-    Custom06 = 0x06,
-    Custom07 = 0x07,
-    Custom08 = 0x08,
-    Mainnet = 0xff,
-}
-
 /// World state of the Themelio blockchain
 #[derive(Debug)]
 pub struct State<C: ContentAddrStore> {
@@ -88,7 +62,7 @@ pub struct State<C: ContentAddrStore> {
 
     pub height: BlockHeight,
     pub history: SmtMapping<C, BlockHeight, Header>,
-    pub coins: SmtMapping<C, CoinID, CoinDataHeight>,
+    pub coins: CoinMapping<C>,
     pub transactions: SmtMapping<C, TxHash, Transaction>,
 
     pub fee_pool: CoinValue,
@@ -141,6 +115,13 @@ impl<C: ContentAddrStore> State<C> {
             || (self.network != NetID::Mainnet && self.network != NetID::Testnet)
     }
 
+    /// Returns true iff TIP 906 rule changes apply.
+    pub fn tip_906(&self) -> bool {
+        false
+        // self.height >= TIP_901_HEIGHT
+        //     || (self.network != NetID::Mainnet && self.network != NetID::Testnet)
+    }
+
     /// Generates an encoding of the state that, in conjunction with a SMT database, can recover the entire state.
     pub fn partial_encoding(&self) -> Vec<u8> {
         let mut out = Vec::new();
@@ -168,7 +149,7 @@ impl<C: ContentAddrStore> State<C> {
             network: header.network,
             height: header.height,
             history: readtree!(header.history_hash),
-            coins: readtree!(header.coins_hash),
+            coins: CoinMapping::new(db.get_tree(header.coins_hash.0).unwrap()),
             transactions: readtree!(header.transactions_hash),
 
             fee_pool: header.fee_pool,
@@ -196,7 +177,15 @@ impl<C: ContentAddrStore> State<C> {
         let network: NetID = readu8!().try_into().unwrap();
         let height = readu64!();
         let history = readtree!();
-        let coins = readtree!();
+        let coins = db
+            .get_tree(
+                read_bts(&mut encoding, 32)
+                    .unwrap()
+                    .as_slice()
+                    .try_into()
+                    .unwrap(),
+            )
+            .unwrap();
         let transactions = readtree!();
 
         let fee_pool = readu128!();
@@ -211,7 +200,7 @@ impl<C: ContentAddrStore> State<C> {
             network,
             height: height.into(),
             history,
-            coins,
+            coins: CoinMapping::new(coins),
             transactions,
 
             fee_pool: fee_pool.into(),
@@ -231,13 +220,13 @@ impl<C: ContentAddrStore> State<C> {
     }
 
     pub fn apply_tx_batch(&mut self, txx: &[Transaction]) -> Result<(), StateError> {
-        let old_hash = self.coins.root_hash();
+        let old_hash = HashVal(self.coins.inner().root_hash());
         StateHandle::new(self).apply_tx_batch(txx)?.commit();
         log::debug!(
             "applied a batch of {} txx to {:?} => {:?}",
             txx.len(),
             old_hash,
-            self.coins.root_hash()
+            HashVal(self.coins.inner().root_hash())
         );
         Ok(())
     }
@@ -245,7 +234,7 @@ impl<C: ContentAddrStore> State<C> {
     /// Finalizes a state into a block. This consumes the state.
     pub fn seal(mut self, action: Option<ProposerAction>) -> SealedState<C> {
         // first apply melmint
-        self = preseal_melmint(self);
+        self = crate::melmint::preseal_melmint(self);
         assert!(self.pools.val_iter().count() >= 2);
 
         let after_tip_901 = self.tip_901();
@@ -286,7 +275,8 @@ impl<C: ContentAddrStore> State<C> {
                 height: self.height,
             };
             // insert the fake coin
-            self.coins.insert(pseudocoin_id, pseudocoin_data);
+            self.coins
+                .insert_coin(pseudocoin_id, pseudocoin_data, self.tip_906());
         }
         // create the finalized state
         SealedState(self, action)
@@ -419,15 +409,6 @@ impl<C: ContentAddrStore> SealedState<C> {
     }
 }
 
-/// ProposerAction describes the standard action that the proposer takes when proposing a block.
-#[derive(Derivative, Serialize, Deserialize, Copy, Clone, Debug, Eq, PartialEq)]
-pub struct ProposerAction {
-    /// Change in fee. This is scaled to the proper size.
-    pub fee_multiplier_delta: i8,
-    /// Where to sweep fees.
-    pub reward_dest: Address,
-}
-
 pub type ConsensusProof = BTreeMap<Ed25519PK, Vec<u8>>;
 
 /// ConfirmedState represents a fully confirmed state with a consensus proof.
@@ -449,53 +430,3 @@ impl<C: ContentAddrStore> ConfirmedState<C> {
         &self.cproof
     }
 }
-
-// impl Deref<Target =
-#[derive(Serialize, Deserialize, Copy, Clone, Debug, Eq, PartialEq, Hash, Arbitrary)]
-/// A block header, which commits to a particular SealedState.
-pub struct Header {
-    pub network: NetID,
-    pub previous: HashVal,
-    pub height: BlockHeight,
-    pub history_hash: HashVal,
-    pub coins_hash: HashVal,
-    pub transactions_hash: HashVal,
-    pub fee_pool: CoinValue,
-    pub fee_multiplier: u128,
-    pub dosc_speed: u128,
-    pub pools_hash: HashVal,
-    pub stakes_hash: HashVal,
-}
-
-impl Header {
-    pub fn hash(&self) -> tmelcrypt::HashVal {
-        tmelcrypt::hash_single(&stdcode::serialize(self).unwrap())
-    }
-}
-
-#[derive(Serialize, Deserialize, Clone, Debug)]
-/// A (serialized) block.
-pub struct Block {
-    pub header: Header,
-    pub transactions: HashSet<Transaction>,
-    pub proposer_action: Option<ProposerAction>,
-}
-
-impl Block {
-    /// Abbreviate a block
-    pub fn abbreviate(&self) -> AbbrBlock {
-        AbbrBlock {
-            header: self.header,
-            txhashes: self.transactions.iter().map(|v| v.hash_nosigs()).collect(),
-            proposer_action: self.proposer_action,
-        }
-    }
-}
-
-/// An abbreviated block
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub struct AbbrBlock {
-    pub header: Header,
-    pub txhashes: BTreeSet<TxHash>,
-    pub proposer_action: Option<ProposerAction>,
-}
